﻿package
{
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.Sprite;
	import flash.events.KeyboardEvent;
	import flash.geom.Point;
	import flash.geom.Rectangle;

	public class Doc extends Sprite
	{
		// Game assets
		private var i:int, j:int, energy:int;
		private var wallList:Array = [];
		private var enemyList:Array = [];
		private var player:Point = new Point();
		private var newPos:Point = new Point();
		
		// Render assets
		private var canvasData:BitmapData = new BitmapData(stage.stageWidth, stage.stageHeight, false, 0x111111);
		private var canvas:Bitmap = new Bitmap(canvasData);
		private var bgData:BitmapData = new BitmapData(stage.stageWidth, stage.stageHeight, false, 0x111111);
		private var wallData:BitmapData = new BitmapData(12, 12, false, 0x111111);
		private var enemyData:BitmapData = new BitmapData(12, 12, false, 0x111111);
		private var playerData:BitmapData = new BitmapData(12, 12, false, 0x111111);
		
		// References
		private var ref:Point;
		
		// Contructor
		public function Doc():void
		{
			// Prepare render assets
			addChild(canvas);
			wallData.fillRect(new Rectangle(3, 0, 12 - 6, 12), 0x335544);
			wallData.fillRect(new Rectangle(0, 3, 12, 12 - 6), 0x335544);
			enemyData.fillRect(new Rectangle(3, 1, 12 - 6, 12 - 2), 0xAA0000);
			enemyData.fillRect(new Rectangle(1, 3, 12 - 2, 12 - 6), 0xAA0000);
			playerData.fillRect(new Rectangle(3, 1, 12 - 6, 12 - 2), 0xDDDDDD);
			playerData.fillRect(new Rectangle(1, 3, 12 - 2, 12 - 6), 0xDDDDDD);
			
			// Start game
			start();
			stage.addEventListener('keyDown', keyDown);
		}
		
		// Capture key input
		private function keyDown(e:KeyboardEvent):void
		{
			// Restart if on win/lose screen
			if (energy < 1 || enemyList.length < 1) {
				start();
				return;
			}
			
			// Get key input
			newPos.x = player.x;
			newPos.y = player.y;
			if (e.keyCode == 37) { // Left
				newPos.x-= 12;
			} else if (e.keyCode == 38) { // Up
				newPos.y-= 12;
			} else if (e.keyCode == 39) { // Right
				newPos.x+= 12;
			} else if (e.keyCode == 40) { // Down
				newPos.y+= 12;
			} else {
				return;
			}
			
			// Collision check against canvas borders
			if (newPos.x < 0 || newPos.x >= 50 * 12 || newPos.y < 0 || newPos.y >= 40 * 12) {
				return;
			}
			
			// Collision check against walls
			if (hit1(newPos, wallList) > -1) {
				return;
			}
			
			// Collision check against enemies
			i = hit1(newPos, enemyList);
			if (i > -1) {
				enemyList.splice(i, 1);
				newPos.x = player.x;
				newPos.y = player.y;
				
				// Player wins
				if (enemyList.length < 1) {
					draw();
					return;
				}
			}
			
			// Move player
			player.x = newPos.x;
			player.y = newPos.y;
			
			// Move enemies
			for (i = 0; i < enemyList.length; i ++) {
				ref = enemyList[i];
				newPos.x = ref.x;
				newPos.y = ref.y;
				
				// Get distance to player
				if ((Math.abs(player.x - ref.x) + Math.abs(player.y - ref.y)) / 12 <= 15) {
					
					// Move towards player
					if (ref.x == player.x) {
						if (ref.y > player.y) {
							newPos.y-= 12;
						} else {
							newPos.y+= 12;
						}
					} else if (ref.y == player.y) {
						if (ref.x > player.x) {
							newPos.x-= 12;
						} else {
							newPos.x+= 12;
						}
					} else {
						if (Math.random() < 0.5) {
							if (ref.x > player.x) {
								newPos.x-= 12;
							} else {
								newPos.x+= 12;
							}
						} else {
							if (ref.y > player.y) {
								newPos.y-= 12;
							} else {
								newPos.y+= 12;
							}
						}
					}
				} else {
					
					// Move randomly
					if (Math.random() < 0.5) {
						if (Math.random() < 0.5) {
							newPos.x-= 12;
						} else {
							newPos.y-= 12;
						}
					} else {
						if (Math.random() < 0.5) {
							newPos.x+= 12;
						} else {
							newPos.y+= 12;
						}
					}
				}
				
				// Collision check against canvas borders
				if (newPos.x < 0 || newPos.x >= 50 * 12 || newPos.y < 0 || newPos.y >= 40 * 12) {
					newPos.x = ref.x;
					newPos.y = ref.y;
				}
				
				// Collision check against walls
				if (hit1(newPos, wallList) > -1) {
					newPos.x = ref.x;
					newPos.y = ref.y;
				}
				
				// Collision check against enemies
				if (hit1(newPos, enemyList) > -1) {
					newPos.x = ref.x;
					newPos.y = ref.y;
				}
				
				// Collision check against player
				if (newPos.x == player.x && newPos.y == player.y) {
					energy --;
					newPos.x = ref.x;
					newPos.y = ref.y;
					
					// Player loses
					if (energy < 1) {
						draw();
						return;
					}
				}
				
				// Move enemy
				ref.x = newPos.x;
				ref.y = newPos.y;
			}
			
			// Re-draw
			draw();
		}
		
		// Start game and place objects
		private function start():void
		{
			// Reset background
			bgData.fillRect(bgData.rect, 0x111111);
			
			// Add walls
			wallList.length = 0;
			for (i = 0; i < 300; i ++) {
				ref = new Point();
				ref.x = int(Math.random() * 50) * 12;
				ref.y = int(Math.random() * 40) * 12;
				wallList[wallList.length] = ref;
				bgData.copyPixels(wallData, wallData.rect, ref);
			}
			
			// Add enemies
			enemyList.length = 0;
			for (i = 0; i < 30; i ++) {
				ref = new Point();
				while (1) {
					ref.x = int(Math.random() * 50) * 12;
					ref.y = int(Math.random() * 40) * 12;
					if (hit1(ref, wallList) == -1) {
						break;
					}
				}
				enemyList[enemyList.length] = ref;
			}
			
			// Add player
			energy = 8;
			while (1) {
				player.x = int(Math.random() * 50) * 12;
				player.y = int(Math.random() * 40) * 12;
				if (hit1(player, wallList) == -1 && hit1(player, enemyList) == -1) {
					break;
				}
			}
			
			draw();
		}
		
		// Draw background and objects
		private function draw():void
		{
			if (energy < 1) {
				
				// Lose screen
				for (i = 0; i < 50; i ++) {
					for (j = 0; j < 40; j ++) {
						newPos.x = i * 12;
						newPos.y = j * 12;
						canvasData.copyPixels(enemyData, enemyData.rect, newPos);
					}
				}
			} else if (enemyList.length < 1) {
				
				// Win screen
				for (i = 0; i < 50; i ++) {
					for (j = 0; j < 40; j ++) {
						newPos.x = i * 12;
						newPos.y = j * 12;
						canvasData.copyPixels(playerData, playerData.rect, newPos);
					}
				}
			} else {
			
				// Reset background
				newPos.x = 0;
				newPos.y = 0;
				canvasData.copyPixels(bgData, bgData.rect, newPos);
				
				// Draw enemies
				for (i = 0; i < enemyList.length; i ++) {
					ref = enemyList[i];
					canvasData.copyPixels(enemyData, enemyData.rect, ref);
				}
				
				// Draw energy bar
				for (i = 0; i < energy; i ++) {
					newPos.x = i * 12 + 12 * 0.25;
					newPos.y = 12 * 0.25;
					canvasData.copyPixels(playerData, playerData.rect, newPos);
				}
				
				// Draw player
				canvasData.copyPixels(playerData, playerData.rect, player);
			}
		}
		
		// Hit test for a single object against a list
		private function hit1(point:Point, list:Array):int
		{
			for (j = 0; j < list.length; j ++) {
				if (point.x == list[j].x && point.y == list[j].y) {
					return j;
				}
			}
			return -1;
		}
	}
}